@@ -1,6 +1,117 @@
 ## SET OF UTILITY FUNCTIONS TO INCLUDE SURVIVAL ANALYSIS RESULTS INTO A HEALTH ECONOMIC MODEL
 ## Gianluca Baio + Will Browne + Peter Konings (10 Jan 2017)
 ##
+
+
+#' Fit parametric survival analysis for health economic evaluations
+#' 
+#' Runs the survival analysis with several useful options, using either MLE
+#' (via flexsurv) or a Bayesian approach (via R-INLA or rstan)
+#' 
+#' On object in the class \code{survHE} containing the following elements
+#' 
+#' @param formula a formula specifying the model to be used, in the form
+#' \code{Surv(time,event)~treatment[+covariates]} for flexsurv, or
+#' \code{inla.surv(time,event)~treatment[+covariates]} for INLA
+#' @param data A data frame containing the data to be used for the analysis.
+#' This must contain data for the 'event' variable. In case there is no
+#' censoring, then \code{event} is a column of 1s.
+#' @param distr a (vector of) string(s) containing the name(s) of the model(s)
+#' to be fitted.  Available options are:
+#' 
+#' \code{flexsurv}:
+#' "exponential","gamma","genf","gengamma","gompertz","weibull",
+#' "weibullPH","loglogistic","lognormal" \code{INLA}:
+#' "exponential","weibull","lognormal","loglogistic" \code{hmc}:
+#' "exponential","gamma","genf","gengamma","gompertz","weibull","weibullPH",
+#' "loglogistic","lognormal"
+#' @param method A string specifying the inferential method (\code{'mle'},
+#' \code{'inla'} or \code{'hmc'}). If \code{method} is set to \code{'hmc'},
+#' then \code{survHE} will write suitable model code in the Stan language
+#' (according to the specified distribution), prepare data and initial values
+#' and then run the model.
+#' @param \dots Additional options (for INLA or HMC).
+#' 
+#' **INLA** specific options \code{dz} = defines the step length for the grid
+#' search over the hyperparameters space (default = 0.1) \code{diff.logdens} =
+#' defines the difference in the log-density for the hyperparameters to stop
+#' integration (default = 5) \code{control.fixed} = defines the default for the
+#' priors, unless specified by the user.  Default values are prior mean = 0 for
+#' *all* fixed effects prior var = 1000 for *all* fixed effects prior mean = 0
+#' for the intercept prior prec -> 0 for the intercept \code{control.family} =
+#' a list of options. If distr is a vector, then can be provided as a named
+#' list of options, for example something like this:
+#' \code{control.family=list(weibull=list(param=c(.1,.1)),lognormal=list(initial=2))}
+#' the names of the elements of the list need to be the same as those given in
+#' the vector \code{distr}
+#' 
+#' **HMC** specific options \code{chains} = number of chains to run in the HMC
+#' (default = 2) \code{iter} = total number of iterations (default = 2000)
+#' \code{warmup} = number of warmup iterations (default = iter/2) \code{thin} =
+#' number of thinning (default = 1) \code{control} = a list specifying
+#' Stan-related options, eg \code{control=list(adapt_delta=0.85)} (default =
+#' NULL) \code{seed} = the random seed (to make things replicable) \code{pars}
+#' = a vector of parameters (string, default = NA) \code{include} = a logical
+#' indicator (if FALSE, then the pars are not saved; default = TRUE)
+#' \code{priors} = a list (of lists) specifying the values for the parameters
+#' of the prior distributions in the models \code{save.stan} = a logical
+#' indicator (default = FALSE). If TRUE, then saves the data list for Stan and
+#' the model file(s)
+#' @return \item{models}{ A list containing the fitted models. These contain
+#' the output from the original inference engine (\code{flexsurv}, \code{INLA}
+#' or \code{rstan}). Can be processed using the methods specific to the
+#' original packages, or via \code{survHE}-specific methods (such as
+#' \code{plot}, \code{print}) or other specialised functions (eg to extrapolate
+#' the survival curves, etc). } \item{model.fitting}{ A list containing the
+#' output of the model-fit statistics (AIC, BIC, DIC). The AIC and BIC are
+#' estimated for all methods, while the DIC is only estimated when using
+#' Bayesian inference. } \item{method}{ A string indicating the method used to
+#' fit the model, ie \code{'mle'}, \code{'inla'} or \code{'hmc'}.  }
+#' \item{misc}{ A list containing the time needed to run the model(s) (in
+#' seconds), the formula used, the results of the Kaplan-Meier analysis (which
+#' is automatically performed using \code{npsurv}) and the original data frame.
+#' }
+#' @note Something will go here
+#' @author Gianluca Baio
+#' @seealso Something will go here
+#' @references Something will go here
+#' @keywords Parametric survival models Bayesian inference via Hamiltonian
+#' Monte Carlo Bayesian inference via Integrated Nested Laplace Approximation
+#' @examples
+#' 
+#' # Loads an example dataset from 'flexsurv'
+#' data(bc)
+#' 
+#' # Fits the same model using the 3 inference methods
+#' mle = fit.models(formula=Surv(recyrs,censrec)~group,data=bc,
+#'     distr="exp",method="mle")
+#' \donttest{
+#' inla = fit.models(formula=Surv(recyrs,censrec)~group,data=bc,
+#'     distr="exp",method="inla")
+#' hmc = fit.models(formula=Surv(recyrs,censrec)~group,data=bc,
+#'     distr="exp",method="hmc")
+#' }
+#' # Prints the results in comparable fashion using the survHE method
+#' print(mle)
+#' \donttest{
+#' print(inla)
+#' print(hmc)
+#' }
+#' 
+#' # Or visualises the results using the original packages methods
+#' print(mle,original=TRUE)
+#' \donttest{
+#' print(inla,original=TRUE)
+#' print(hmc,original=TRUE)
+#' }
+#' 
+#' # Plots the survival curves and estimates
+#' plot(mle)
+#' \donttest{
+#' plot(mle,inla,hmc,labs=c("MLE","INLA","HMC"),colors=c("black","red","blue"))
+#' }
+#' 
+#' @export fit.models
 fit.models <- function(formula=NULL,data,distr=NULL,method="mle",...) {
   ## Main function - runs the survival analysis with several useful options
   ## formula = a formula specifying the model to be used, in the form
@@ -313,16 +424,16 @@ fit.models <- function(formula=NULL,data,distr=NULL,method="mle",...) {
 
     # Now runs the model
     ## Stan options (the defaults are set in line with Stan's original)
-    nlist <- NULL
+    #nlist <- NULL
     if(exists("chains",where=exArgs)) {chains <- exArgs$chains} else {chains <- 2} # DO WE WANT 4???
     if(exists("iter",where=exArgs)) {iter <- exArgs$iter} else {iter <- 2000}
     if(exists("warmup",where=exArgs)) {warmup <- exArgs$warmup} else {warmup <- floor(iter/2)}
     if(exists("thin",where=exArgs)) {thin <- exArgs$thin} else {thin <- 1}
     if(exists("control",where=exArgs)) {
-        check <- unlist(lapply(1:length(control),function(i) class(control[[i]])))
+        check <- unlist(lapply(1:length(exArgs$control),function(i) class(exArgs$control[[i]])))
         nlists <- length(check)
         if (nlists==length(distr)) {
-            control <- ifelse(nlist==1,list(exArgs$control),exArgs$control)
+           control <- ifelse(nlists==1,list(exArgs$control),exArgs$control)
         } 
         if (nlists<length(check)) {
             control <- list(exArgs$control,replicate((nlists-1),list(NULL)))
@@ -342,6 +453,7 @@ fit.models <- function(formula=NULL,data,distr=NULL,method="mle",...) {
     if(exists("include",where=exArgs)) {include <- exArgs$include} else {include <- FALSE}
     if(exists("k",where=exArgs)) {k <- exArgs$k} else {k <- 0}
     if(exists("cores",where=exArgs)) {cores <- exArgs$cores} else {cores <- 1}
+    if(exists("init",where=exArgs)) {init <- exArgs$init} else {init="random"}
 
     non.on.log.scale <- c("genf","gengamma","lognormal")
     
@@ -534,7 +646,7 @@ fit.models <- function(formula=NULL,data,distr=NULL,method="mle",...) {
       # Now runs Stan to sample from the posterior distributions
       tic <- proc.time()
       out <- rstan::sampling(dso[[x]],data.stan,chains=chains,iter=iter,warmup=warmup,thin=thin,seed=seed,control=control[[x]],
-                      pars=pars,include=include,cores=cores)
+                      pars=pars,include=include,cores=cores,init=init)
       toc <- proc.time()-tic
       time2run <- toc[3]
       list(out=out,data.stan=data.stan,time2run=time2run)
@@ -598,8 +710,8 @@ fit.models <- function(formula=NULL,data,distr=NULL,method="mle",...) {
 	      shape <- as.numeric(rstan::extract(mod[[i]]$out)$alpha)
 	      shape.hat=median(shape)
 	      logf <- matrix(unlist(lapply(1:nrow(linpred),function(i) {
-	        data.stan$d*log(hgompertz(data.stan$t,shape[i],exp(linpred[i,]))) + 
-	          log(1-pgompertz(data.stan$t,shape[i],exp(linpred[i,])))
+	        data.stan$d*log(hgompertz(data.stan$t,shape=shape[i],rate=exp(linpred[i,]))) + 
+	          log(1-pgompertz(shape=data.stan$t,shape[i],rate=exp(linpred[i,])))
 	      })),nrow=nrow(linpred),byrow=T)
 	      logf.hat <- matrix(data.stan$d*log(hgompertz(data.stan$t,shape.hat,exp(linpred.hat)))+
 	                          log(1-pgompertz(data.stan$t,shape.hat,exp(linpred.hat))),nrow=1)
